@@ -25,6 +25,8 @@ void lookup_gtabn(char *ch, char *out);
 char *htmlspecialchars(char *s, char out[]);
 void hide_gtab_pre_sel();
 
+extern gint64 key_press_time_ctrl;
+
 extern gboolean test_mode;
 
 GEDIT *gbuf;
@@ -1058,6 +1060,9 @@ void gtab_scan_pre_select(gboolean b_incr)
 
   int len, selN, pre_selN=0, max_len=-1, max_selN;
   for(len=1; len <= Maxlen; len++) {
+    int idx = ggg.gbufN - len;
+    if (gbuf[idx].flag & FLAG_CHPHO_PHRASE_TAIL)
+      break;
     int mlen = scanphr_e(ggg.gbufN - len, len, b_incr, &selN);
     if (mlen) {
       max_len = len;
@@ -1088,10 +1093,11 @@ void gtab_scan_pre_select(gboolean b_incr)
   char tt[4096];
   tt[0]=0;
   int i;
-  char *br=gtab_vertical_select?"\n":"";
 
   for(i=0;i<tss.pre_selN; i++) {
     char ts[(MAX_PHRASE_LEN+3) * CH_SZ + 1];
+    char *br= (i < tss.pre_selN-1 && gtab_vertical_select)?"\n":"";
+
     sprintf(ts, "<span foreground=\"%s\">%c</span>%s%s", gcin_sel_key_color,
       cur_inmd->selkey[i], tss.pre_sel[i].str, br);
     strcat(tt, ts);
@@ -1106,14 +1112,8 @@ void gtab_scan_pre_select(gboolean b_incr)
 
 int shift_key_idx(char *s, KeySym xkey);
 
-
-gboolean gtab_pre_select_shift(KeySym key, int kbstate)
+gboolean gtab_pre_select_idx(int c)
 {
-//  dbg("gtab_pre_select_shift %c\n", key);
-  if (!gtab_phrase_pre_select || !tss.pre_selN)
-    return FALSE;
-
-  int c = shift_key_idx(cur_inmd->selkey, key);
   if (c < 0)
     return FALSE;
   if (c >= tss.pre_selN)
@@ -1125,7 +1125,44 @@ gboolean gtab_pre_select_shift(KeySym key, int kbstate)
 #endif
 
   gtab_buf_backspaceN(gtab_pre_select_phrase_len);
-  insert_gbuf_cursor_phrase(tss.pre_sel[c].str, tss.pre_sel[c].phkey, tss.pre_sel[c].len);
+  int len = tss.pre_sel[c].len;
+  insert_gbuf_cursor_phrase(tss.pre_sel[c].str, tss.pre_sel[c].phkey, len);
+  gbuf[ggg.gbufN-1].flag |= FLAG_CHPHO_PHRASE_TAIL;
+
   hide_gtab_pre_sel();
   return TRUE;
 }
+
+
+gboolean gtab_pre_select_shift(KeySym key, int kbstate)
+{
+//  dbg("gtab_pre_select_shift %c\n", key);
+  if (!gtab_phrase_pre_select || !tss.pre_selN)
+    return FALSE;
+
+  int c = shift_key_idx(cur_inmd->selkey, key);
+  return gtab_pre_select_idx(c);
+}
+
+
+int feedkey_gtab_release(KeySym xkey, int kbstate)
+{
+  gint64 kpt;
+
+  switch (xkey) {
+     case XK_Control_L:
+     case XK_Control_R:
+       kpt = key_press_time_ctrl;
+       key_press_time_ctrl = 0;
+        if (current_time() - kpt < 300000 && tss.pre_selN) {
+          if (!test_mode) {
+            tss.ctrl_pre_sel = TRUE;
+          }
+          return 1;
+        } else
+          return 0;
+     default:
+        return 0;
+  }
+}
+
