@@ -20,6 +20,7 @@ GTAB_ST ggg = {.sel1st_i=MAX_SELKEY - 1};
 GTAB_ST ggg;
 #endif
 
+extern gint64 key_press_time_ctrl;
 extern GtkWidget *gwin_gtab;
 void hide_gtab_pre_sel();
 void gtab_scan_pre_select(gboolean);
@@ -1343,6 +1344,7 @@ void save_gtab_buf_phrase(KeySym key);
 gboolean save_gtab_buf_shift_enter();
 gboolean win_sym_page_up(), win_sym_page_down();
 u_int64_t vmaskci;
+gboolean gtab_pre_select_idx(int c);
 
 gboolean feedkey_gtab(KeySym key, int kbstate)
 {
@@ -1357,11 +1359,6 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
 
   bzero(seltab_phrase, sizeof(seltab_phrase));
 
-#if 0
-  if (key>= XK_KP_0 && key<= XK_KP_9)
-	dbg("uuuu");
-#endif
-
 //  dbg("uuuuu %x %x   shift,ctrl:%d,%d\n", key, kbstate, shift_m, ctrl_m);
 
   if (!cur_inmd)
@@ -1376,6 +1373,13 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
   if (ggg.gbufN && key==XK_Tab)
     return 1;
 
+  if ((key==XK_Control_L||key==XK_Control_R) && !key_press_time_ctrl && tss.pre_selN) {
+    key_press_time_ctrl = current_time();
+    return TRUE;
+  } else {
+    key_press_time_ctrl = 0;
+  }
+
   if (kbstate & (Mod1Mask|Mod4Mask|Mod5Mask|ControlMask)) {
     return 0;
   }
@@ -1387,7 +1391,6 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
     GTK_WIDGET_VISIBLE(gwin_pho))
      hide_win_pho();
 
-
   if (gtab_capslock_in_eng && (kbstate&LockMask) && !BITON(cur_inmd->flag, FLAG_KEEP_KEY_CASE)) {
     if (key < 0x20 || key>=0x7f)
       goto shift_proc;
@@ -1741,6 +1744,9 @@ direct_select:
     default:
 next:
 
+      if (shift_m && tss.pre_selN && shift_char_proc(key, kbstate))
+        return TRUE;
+
       clear_gtab_input_error_color();
 
       if (ggg.invalid_spc && gtab_invalid_key_in) {
@@ -1748,13 +1754,12 @@ next:
       }
       if (key>=XK_KP_0 && key<=XK_KP_9) {
         if (!ggg.ci) {
-		  if (ggg.gbufN) {
+          if (ggg.gbufN) {
             insert_gbuf_cursor_char(key - XK_KP_0 + '0');
-			return 1;
-		  }
-		  else
+            return 1;
+          } else
             return 0;
-		}
+        }
         if (!strncmp(cur_inmd->filename, "dayi", 4)) {
           key = key - XK_KP_0 + '0';
           is_keypad = TRUE;
@@ -1795,17 +1800,21 @@ keypad_proc:
       }
 #endif
 
-	  if (key < 0x7f)
-		inkey= cur_inmd->keymap[key];
-	  else
-		inkey = 0;
 
-//	  dbg("ggg.spc_pressed %d %d %d is_keypad:%d\n", ggg.spc_pressed, ggg.last_full, cur_inmd->MaxPress, is_keypad);
+      if (key < 0x7f)
+        inkey= cur_inmd->keymap[key];
+      else
+        inkey = 0;
+
+//        dbg("ggg.spc_pressed %d %d %d is_keypad:%d\n", ggg.spc_pressed, ggg.last_full, cur_inmd->MaxPress, is_keypad);
 
 #if 1 // for dayi, testcase :  6 space keypad6
+      int vv = pselkey - cur_inmd->selkey;
+      if (pselkey && tss.ctrl_pre_sel) {
+        if (gtab_pre_select_idx(vv))
+          return TRUE;
+      } else
       if (( (ggg.spc_pressed||ggg.last_full||is_keypad) ||(ggg.wild_mode && (!inkey ||pendkey)) || ggg.gtab_buf_select) && pselkey) {
-        int vv = pselkey - cur_inmd->selkey;
-
         if ((_gtab_space_auto_first & GTAB_space_auto_first_any) && !ggg.wild_mode)
           vv++;
 
